@@ -1,4 +1,4 @@
-/*	$OpenBSD$	*/
+/*	$OpenBSD: filter.c,v 1.16 2016/02/03 11:14:08 eric Exp $	*/
 
 /*
  * Copyright (c) 2011 Gilles Chehade <gilles@poolp.org>
@@ -43,14 +43,8 @@
 #include "smtpd.h"
 #include "log.h"
 
-enum {
-	QK_QUERY,
-	QK_EVENT,
-};
-
 enum {
 	QUERY_READY,
-	QUERY_WAITING,
 	QUERY_RUNNING,
 	QUERY_DONE
 };
@@ -77,8 +71,6 @@ struct filter_session {
 	struct filter_lst	*filters;
 	struct filter		*fcurr;
 
-	struct filter_query_lst	 queries;
-
 	int			 error;
 	struct io		 iev;
 	struct iobuf		 ibuf;
@@ -90,13 +82,10 @@ struct filter_session {
 
 struct filter_query {
 	uint64_t			 qid;
-	int				 kind;
 	int				 type;
 	struct filter_session		*session;
-	TAILQ_ENTRY(filter_query)	 entry;
 
 	int				 state;
-	int				 hasrun;
 	struct filter			*current;
 
 	/* current data */
@@ -120,7 +109,8 @@ struct filter_query {
 };
 
 static void filter_imsg(struct mproc *, struct imsg *);
-static struct filter_query *filter_query(struct filter_session *, int, int);
+static void filter_post_event(uint64_t, int, struct filter *, struct filter *);
+static struct filter_query *filter_query(struct filter_session *, int);
 static void filter_drain_query(struct filter_query *);
 static void filter_run_query(struct filter *, struct filter_query *);
 static void filter_end_query(struct filter_query *);
@@ -135,7 +125,6 @@ static const char * filter_session_to_text(struct filter_session *);
 static const char * filter_query_to_text(struct filter_query *);
 static const char * filter_to_text(struct filter *);
 static const char * filter_proc_to_text(struct filter_proc *);
-static const char * kind_to_str(int);
 static const char * query_to_str(int);
 static const char * event_to_str(int);
 static const char * status_to_str(int);
@@ -144,6 +133,16 @@ static const char * filterimsg_to_str(int);
 struct tree	sessions;
 struct tree	queries;
 
+static void
+filter_add_arg(struct filter_conf *filter, char *arg)
+{
+	if (filter->argc == MAX_FILTER_ARGS) {
+		log_warnx("warn: filter \"%s\" is full", filter->name);
+		fatalx("exiting");
+	}
+	filter->argv[filter->argc++] = arg;
+}
+
 static void
 filter_extend_chain(struct filter_lst *chain, const char *name)
 {
@@ -154,12 +153,12 @@ filter_extend_chain(struct filter_lst *chain, const char *name)
 
 	fconf = dict_xget(&env->sc_filters, name);
 	if (fconf->chain) {
-		log_debug("filter:     extending with \"%s\"", name);
+		log_debug("filter: extending with \"%s\"", name);
 		for (i = 0; i < fconf->argc; i++)
 			filter_extend_chain(chain, fconf->argv[i]);
 	}
 	else {
-		log_debug("filter:     adding filter \"%s\"", name);
+		log_debug("filter: adding filter \"%s\"", name);
 		n = xcalloc(1, sizeof(*n), "filter_extend_chain");
 		fchain = dict_get(&chains, name);
 		n->proc = TAILQ_FIRST(fchain)->proc;
@@ -195,18 +194,20 @@ filter_postfork(void)
 			continue;
 
 		log_debug("filter: building simple chain \"%s\"", filter->name);
-
 		proc = xcalloc(1, sizeof(*proc), "filter_postfork");
 		p = &proc->mproc;
 		p->handler = filter_imsg;
 		p->proc = PROC_FILTER;
 		p->name = xstrdup(filter->name, "filter_postfork");
 		p->data = proc;
+		if (verbose & TRACE_DEBUG)
+			filter_add_arg(filter, "-v");
+		if (foreground_log)
+			filter_add_arg(filter, "-d");
 		if (mproc_fork(p, filter->path, filter->argv) < 0)
 			fatalx("filter_postfork");
 
 		log_debug("filter: registering proc \"%s\"", filter->name);
-
 		f = xcalloc(1, sizeof(*f), "filter_postfork");
 		f->proc = proc;
 
@@ -225,7 +226,8 @@ filter_postfork(void)
 	while (!done) {
 		done = 1;
 		iter = NULL;
-		while (dict_iter(&env->sc_filters, &iter, NULL, (void **)&filter)) {
+		while (dict_iter(&env->sc_filters, &iter, NULL,
+		    (void **)&filter)) {
 			if (filter->done)
 				continue;
 			done = 0;
@@ -240,10 +242,12 @@ filter_postfork(void)
 				continue;
 			fchain = xcalloc(1, sizeof(*fchain), "filter_postfork");
 			TAILQ_INIT(fchain);
-			log_debug("filter: building chain \"%s\"...", filter->name);
+			log_debug("filter: building chain \"%s\"...",
+			    filter->name);
 			for (i = 0; i < filter->argc; i++)
 				filter_extend_chain(fchain, filter->argv[i]);
-			log_debug("filter: done building chain \"%s\"", filter->name);
+			log_debug("filter: done building chain \"%s\"",
+			    filter->name);
 			dict_xset(&chains, filter->name, fchain);
 		}
 	}
@@ -283,7 +287,6 @@ void
 filter_event(uint64_t id, int event)
 {
 	struct filter_session	*s;
-	struct filter_query	*q;
 
 	if (event == EVENT_DISCONNECT)
 		/* On disconnect, the session is virtualy dead */
@@ -291,9 +294,15 @@ filter_event(uint64_t id, int event)
 	else
 		s = tree_xget(&sessions, id);
 
-	q = filter_query(s, QK_EVENT, event);
+	filter_post_event(id, event, TAILQ_FIRST(s->filters), NULL);
 
-	filter_drain_query(q);
+	if (event == EVENT_DISCONNECT) {
+		io_clear(&s->iev);
+		iobuf_clear(&s->ibuf);
+		if (s->ofile)
+			fclose(s->ofile);
+		free(s);
+	}
 }
 
 void
@@ -309,12 +318,10 @@ filter_connect(uint64_t id, const struct sockaddr *local,
 		filter = "<no-filter>";
 	s->filters = dict_xget(&chains, filter);
 	s->iev.sock = -1;
-	TAILQ_INIT(&s->queries);
 	tree_xset(&sessions, s->id, s);
 
 	filter_event(id, EVENT_CONNECT);
-
-	q = filter_query(s, QK_QUERY, QUERY_CONNECT);
+	q = filter_query(s, QUERY_CONNECT);
 
 	memmove(&q->u.connect.local, local, SA_LEN(local));
 	memmove(&q->u.connect.remote, remote, SA_LEN(remote));
@@ -334,7 +341,7 @@ filter_mailaddr(uint64_t id, int type, const struct mailaddr *maddr)
 	struct filter_query	*q;
 
 	s = tree_xget(&sessions, id);
-	q = filter_query(s, QK_QUERY, type);
+	q = filter_query(s, type);
 
 	strlcpy(q->u.maddr.user, maddr->user, sizeof(q->u.maddr.user));
 	strlcpy(q->u.maddr.domain, maddr->domain, sizeof(q->u.maddr.domain));
@@ -349,7 +356,7 @@ filter_line(uint64_t id, int type, const char *line)
 	struct filter_query	*q;
 
 	s = tree_xget(&sessions, id);
-	q = filter_query(s, QK_QUERY, type);
+	q = filter_query(s, type);
 
 	if (line)
 		strlcpy(q->u.line, line, sizeof(q->u.line));
@@ -364,7 +371,7 @@ filter_eom(uint64_t id, int type, size_t datalen)
 	struct filter_query	*q;
 
 	s = tree_xget(&sessions, id);
-	q = filter_query(s, QK_QUERY, type);
+	q = filter_query(s, type);
 	q->u.datalen = datalen;
 
 	filter_drain_query(q);
@@ -375,9 +382,10 @@ filter_set_sink(struct filter_session *s, int sink)
 {
 	struct mproc	*p;
 
-	while(s->fcurr) {
+	while (s->fcurr) {
 		if (s->fcurr->proc->hooks & HOOK_DATALINE) {
-			log_trace(TRACE_FILTERS, "filter: sending fd %d to %s", sink, filter_to_text(s->fcurr));
+			log_trace(TRACE_FILTERS, "filter: sending fd %d to %s",
+			    sink, filter_to_text(s->fcurr));
 			p = &s->fcurr->proc->mproc;
 			m_create(p, IMSG_FILTER_PIPE, 0, 0, sink);
 			m_add_id(p, s->id);
@@ -388,7 +396,6 @@ filter_set_sink(struct filter_session *s, int sink)
 	}
 
 	log_trace(TRACE_FILTERS, "filter: chain input is %d", sink);
-
 	smtp_filter_fd(s->id, sink);
 }
 
@@ -405,28 +412,34 @@ filter_build_fd_chain(uint64_t id, int sink)
 	filter_set_sink(s, fd);
 }
 
+void
+filter_post_event(uint64_t id, int event, struct filter *f, struct filter *end)
+{
+	for(; f && f != end; f = TAILQ_NEXT(f, entry)) {
+		log_trace(TRACE_FILTERS, "filter: post-event event=%s filter=%s",
+		    event_to_str(event), f->proc->mproc.name);
+
+		m_create(&f->proc->mproc, IMSG_FILTER_EVENT, 0, 0, -1);
+		m_add_id(&f->proc->mproc, id);
+		m_add_int(&f->proc->mproc, event);
+		m_close(&f->proc->mproc);
+	}
+}
+
 static struct filter_query *
-filter_query(struct filter_session *s, int kind, int type)
+filter_query(struct filter_session *s, int type)
 {
 	struct filter_query	*q;
 
 	q = xcalloc(1, sizeof(*q), "filter_query");
 	q->qid = generate_uid();
 	q->session = s;
-	q->kind = kind;
 	q->type = type;
-	TAILQ_INSERT_TAIL(&s->queries, q, entry);
 
 	q->state = QUERY_READY;
 	q->current = TAILQ_FIRST(s->filters);
-	q->hasrun = 0;
 
-	if (kind == QK_QUERY)
-		log_trace(TRACE_FILTERS, "filter: new query %s %s", kind_to_str(kind),
-		    query_to_str(type));
-	else
-		log_trace(TRACE_FILTERS, "filter: new query %s %s", kind_to_str(kind),
-		    event_to_str(type));
+	log_trace(TRACE_FILTERS, "filter: new query %s", query_to_str(type));
 
 	return (q);
 }
@@ -434,52 +447,29 @@ filter_query(struct filter_session *s, int kind, int type)
 static void
 filter_drain_query(struct filter_query *q)
 {
-	struct filter_query	*prev;
-
-	log_trace(TRACE_FILTERS, "filter: filter_drain_query %s", filter_query_to_text(q));
+	log_trace(TRACE_FILTERS, "filter: filter_drain_query %s",
+	    filter_query_to_text(q));
 
 	/*
 	 * The query must be passed through all filters that registered
 	 * a hook, until one rejects it.
 	 */
 	while (q->state != QUERY_DONE) {
-
 		/* Walk over all filters */
 		while (q->current) {
-
-			/* Trigger the current filter if not done yet. */
-			if (!q->hasrun) {
-				filter_run_query(q->current, q);
-				q->hasrun = 1;
-			}
+			filter_run_query(q->current, q);
 			if (q->state == QUERY_RUNNING) {
 				log_trace(TRACE_FILTERS,
 				    "filter: waiting for running query %s",
 				    filter_query_to_text(q));
 				return;
 			}
-
-			/*
-			 * Do not move forward if the query ahead of us is
-			 * waiting on this filter.
-			 */
-			prev = TAILQ_PREV(q, filter_query_lst, entry);
-			if (prev && prev->current == q->current) {
-				q->state = QUERY_WAITING;
-				log_trace(TRACE_FILTERS,
-				    "filter: query blocked by previous query %s",
-				    filter_query_to_text(prev));
-				return;
-			}
-
-			q->current = TAILQ_NEXT(q->current, entry);
-			q->hasrun = 0;
 		}
 		q->state = QUERY_DONE;
 	}
 
 	/* Defer the response if the file is not closed yet. */
-	if (q->kind == QK_QUERY && q->type == QUERY_EOM && q->session->ofile) {
+	if (q->type == QUERY_EOM && q->session->ofile && q->smtp.status == FILTER_OK) {
 		log_debug("filter: deferring eom query...");
 		q->session->eom = q;
 		return;
@@ -491,79 +481,59 @@ filter_drain_query(struct filter_query *q)
 static void
 filter_run_query(struct filter *f, struct filter_query *q)
 {
-	if (q->kind == QK_QUERY) {
+	log_trace(TRACE_FILTERS,
+	    "filter: running filter %s for query %s",
+	    filter_to_text(f), filter_query_to_text(q));
 
-		log_trace(TRACE_FILTERS, "filter: running filter %s for query %s",
-		    filter_to_text(f), filter_query_to_text(q));
+	m_create(&f->proc->mproc, IMSG_FILTER_QUERY, 0, 0, -1);
+	m_add_id(&f->proc->mproc, q->session->id);
+	m_add_id(&f->proc->mproc, q->qid);
+	m_add_int(&f->proc->mproc, q->type);
 
-		m_create(&f->proc->mproc, IMSG_FILTER_QUERY, 0, 0, -1);
-		m_add_id(&f->proc->mproc, q->session->id);
-		m_add_id(&f->proc->mproc, q->qid);
-		m_add_int(&f->proc->mproc, q->type);
-
-		switch (q->type) {
-		case QUERY_CONNECT:
-			m_add_sockaddr(&f->proc->mproc,
-			    (struct sockaddr *)&q->u.connect.local);
-			m_add_sockaddr(&f->proc->mproc,
-			    (struct sockaddr *)&q->u.connect.remote);
-			m_add_string(&f->proc->mproc, q->u.connect.hostname);
-			break;
-		case QUERY_HELO:
-			m_add_string(&f->proc->mproc, q->u.line);
-			break;
-		case QUERY_MAIL:
-		case QUERY_RCPT:
-			m_add_mailaddr(&f->proc->mproc, &q->u.maddr);
-			break;
-		case QUERY_EOM:
-			m_add_u32(&f->proc->mproc, q->u.datalen);
-			break;
-		default:
-			break;
-		}
-		m_close(&f->proc->mproc);
-
-		tree_xset(&queries, q->qid, q);
-		q->state = QUERY_RUNNING;
+	switch (q->type) {
+	case QUERY_CONNECT:
+		m_add_sockaddr(&f->proc->mproc,
+		    (struct sockaddr *)&q->u.connect.local);
+		m_add_sockaddr(&f->proc->mproc,
+		    (struct sockaddr *)&q->u.connect.remote);
+		m_add_string(&f->proc->mproc, q->u.connect.hostname);
+		break;
+	case QUERY_HELO:
+		m_add_string(&f->proc->mproc, q->u.line);
+		break;
+	case QUERY_MAIL:
+	case QUERY_RCPT:
+		m_add_mailaddr(&f->proc->mproc, &q->u.maddr);
+		break;
+	case QUERY_EOM:
+		m_add_u32(&f->proc->mproc, q->u.datalen);
+		break;
+	default:
+		break;
 	}
-	else {
-		log_trace(TRACE_FILTERS, "filter: running filter %s for query %s",
-		    filter_to_text(f), filter_query_to_text(q));
+	m_close(&f->proc->mproc);
 
-		m_create(&f->proc->mproc, IMSG_FILTER_EVENT, 0, 0, -1);
-		m_add_id(&f->proc->mproc, q->session->id);
-		m_add_int(&f->proc->mproc, q->type);
-		m_close(&f->proc->mproc);
- 	}
+	tree_xset(&queries, q->qid, q);
+	q->state = QUERY_RUNNING;
 }
 
 static void
 filter_end_query(struct filter_query *q)
 {
 	struct filter_session *s = q->session;
+	const char *response = q->smtp.response;
 
-	log_trace(TRACE_FILTERS, "filter: filter_end_query %s", filter_query_to_text(q));
+	log_trace(TRACE_FILTERS, "filter: filter_end_query %s",
+	    filter_query_to_text(q));
 
-	if (q->kind == QK_EVENT)
-		goto done;
-
-	if (q->type == QUERY_EOM) {
-
-		log_trace(TRACE_FILTERS, "filter: filter_end_query(%d, %zu, %zu)", s->iev.sock,
-		    s->idatalen, q->u.datalen);
-
-		if (s->error) {
-			smtp_filter_response(s->id, QUERY_EOM, FILTER_FAIL, 0, NULL);
-			free(q->smtp.response);
-			goto done;
-		}
-		else if (q->u.datalen != s->idatalen) {
-			log_warnx("filter: datalen mismatch on session %" PRIx64
-			    ": %zu/%zu", s->id, s->idatalen, q->u.datalen);
-			smtp_filter_response(s->id, QUERY_EOM, FILTER_FAIL, 0, NULL);
-			free(q->smtp.response);
-			goto done;
+	if (q->type == QUERY_EOM && q->smtp.status == FILTER_OK) {
+		if (s->error || q->u.datalen != s->idatalen) {
+			response = "Internal error";
+			q->smtp.code = 451;
+			q->smtp.status = FILTER_FAIL;
+			if (!s->error)
+				log_warnx("filter: datalen mismatch on session %" PRIx64
+				    ": %zu/%zu", s->id, s->idatalen, q->u.datalen);
 		}
 	}
 
@@ -573,18 +543,11 @@ filter_end_query(struct filter_query *q)
 	    q->qid,
 	    status_to_str(q->smtp.status),
 	    q->smtp.code,
-	    q->smtp.response);
-	smtp_filter_response(s->id, q->type, q->smtp.status, q->smtp.code,
-	    q->smtp.response);
-	free(q->smtp.response);
+	    response);
 
-    done:
-	TAILQ_REMOVE(&s->queries, q, entry);
-	if (q->kind == QK_EVENT && q->type == EVENT_DISCONNECT) {
-		io_clear(&s->iev);
-		iobuf_clear(&s->ibuf);
-		free(s);
-	}
+	smtp_filter_response(s->id, q->type, q->smtp.status, q->smtp.code,
+	    response);
+	free(q->smtp.response);
 	free(q);
 }
 
@@ -593,7 +556,7 @@ filter_imsg(struct mproc *p, struct imsg *imsg)
 {
 	struct filter_proc	*proc = p->data;
 	struct filter_session	*s;
-	struct filter_query	*q, *next;
+	struct filter_query	*q;
 	struct msg		 m;
 	const char		*line;
 	uint64_t		 qid;
@@ -650,7 +613,8 @@ filter_imsg(struct mproc *p, struct imsg *imsg)
 
 		q = tree_xpop(&queries, qid);
 		if (q->type != type) {
-			log_warnx("warn: filter: type mismatch %d != %d", q->type, type);
+			log_warnx("warn: filter: type mismatch %d != %d",
+			    q->type, type);
 			fatalx("exiting");
 		}
 		q->smtp.status = status;
@@ -664,15 +628,8 @@ filter_imsg(struct mproc *p, struct imsg *imsg)
 		if (type == QUERY_EOM)
 			q->u.datalen = datalen;
 
-		next = TAILQ_NEXT(q, entry);
+		q->current = TAILQ_NEXT(q->current, entry);
 		filter_drain_query(q);
-
-		/*
-		 * If there is another query after this one which is waiting,
-		 * make it move forward.
-		 */
-		if (next && next->state == QUERY_WAITING)
-			filter_drain_query(next);
 		break;
 
 	case IMSG_FILTER_PIPE:
@@ -729,14 +686,16 @@ filter_tx_io(struct io *io, int evt)
 	size_t			 len, n;
 	char			*data;
 
-	log_trace(TRACE_FILTERS, "filter: filter_tx_io(%p, %s)", s, io_strevent(evt));
+	log_trace(TRACE_FILTERS, "filter: filter_tx_io(%p, %s)",
+	    s, io_strevent(evt));
 
 	switch (evt) {
 	case IO_DATAIN:
 		data = iobuf_data(&s->ibuf);
 		len = iobuf_len(&s->ibuf);
 
-		log_trace(TRACE_FILTERS, "filter: filter_tx_io: datain (%zu) for req %016"PRIx64"",
+		log_trace(TRACE_FILTERS,
+		    "filter: filter_tx_io: datain (%zu) for req %016"PRIx64"",
 		    len, s->id);
 
 		n = fwrite(data, 1, len, s->ofile);
@@ -751,11 +710,14 @@ filter_tx_io(struct io *io, int evt)
 		return;
 
 	case IO_DISCONNECTED:
-		log_trace(TRACE_FILTERS, "debug: filter: tx done (%zu) for req %016"PRIx64, s->idatalen, s->id);
+		log_trace(TRACE_FILTERS,
+		    "debug: filter: tx done (%zu) for req %016"PRIx64,
+		    s->idatalen, s->id);
 		break;
 
 	default:
-		log_warn("warn: filter_tx_io: bad evt (%d) for req %016"PRIx64, evt, s->id);
+		log_warn("warn: filter_tx_io: bad evt (%d) for req %016"PRIx64,
+		    evt, s->id);
 		s->error = 1;
 		break;
 	}
@@ -782,37 +744,32 @@ filter_query_to_text(struct filter_query *q)
 
 	tmp[0] = '\0';
 
-	if (q->kind == QK_QUERY) {
-		switch(q->type) {
-		case QUERY_CONNECT:
-			strlcat(tmp, "=", sizeof tmp);
-			strlcat(tmp, ss_to_text(&q->u.connect.local), sizeof tmp);
-			strlcat(tmp, " <-> ", sizeof tmp);
-			strlcat(tmp, ss_to_text(&q->u.connect.remote), sizeof tmp);
-			strlcat(tmp, "(", sizeof tmp);
-			strlcat(tmp, q->u.connect.hostname, sizeof tmp);
-			strlcat(tmp, ")", sizeof tmp);
-			break;
-		case QUERY_MAIL:
-		case QUERY_RCPT:
-			snprintf(tmp, sizeof tmp, "=%s@%s",
-			    q->u.maddr.user, q->u.maddr.domain);
-			break;
-		case QUERY_HELO:
-			snprintf(tmp, sizeof tmp, "=%s", q->u.line);
-			break;
-		default:
-			break;
-		}
-		snprintf(buf, sizeof buf, "%016"PRIx64"[%s,%s%s,%s]",
-		    q->qid, kind_to_str(q->kind), query_to_str(q->type), tmp,
-		    filter_session_to_text(q->session));
-	}
-	else {
-		snprintf(buf, sizeof buf, "%016"PRIx64"[%s,%s%s,%s]",
-		    q->qid, kind_to_str(q->kind), event_to_str(q->type), tmp,
-		    filter_session_to_text(q->session));
+	switch (q->type) {
+	case QUERY_CONNECT:
+		strlcat(tmp, "=", sizeof tmp);
+		strlcat(tmp, ss_to_text(&q->u.connect.local),
+		    sizeof tmp);
+		strlcat(tmp, " <-> ", sizeof tmp);
+		strlcat(tmp, ss_to_text(&q->u.connect.remote),
+		    sizeof tmp);
+		strlcat(tmp, "(", sizeof tmp);
+		strlcat(tmp, q->u.connect.hostname, sizeof tmp);
+		strlcat(tmp, ")", sizeof tmp);
+		break;
+	case QUERY_MAIL:
+	case QUERY_RCPT:
+		snprintf(tmp, sizeof tmp, "=%s@%s",
+		    q->u.maddr.user, q->u.maddr.domain);
+		break;
+	case QUERY_HELO:
+		snprintf(tmp, sizeof tmp, "=%s", q->u.line);
+		break;
+	default:
+		break;
 	}
+	snprintf(buf, sizeof buf, "%016"PRIx64"[%s%s,%s]",
+	    q->qid, query_to_str(q->type), tmp,
+	    filter_session_to_text(q->session));
 
 	return (buf);
 }
@@ -825,7 +782,8 @@ filter_session_to_text(struct filter_session *s)
 	if (s == NULL)
 		return "filter_session@NULL";
 
-	snprintf(buf, sizeof(buf), "filter_session@%p[datalen=%zu,eom=%p,ofile=%p]",
+	snprintf(buf, sizeof(buf),
+	    "filter_session@%p[datalen=%zu,eom=%p,ofile=%p]",
 	    s, s->idatalen, s->eom, s->ofile);
 
 	return buf;
@@ -898,17 +856,6 @@ event_to_str(int event)
 	}
 }
 
-static const char *
-kind_to_str(int type)
-{
-	switch (type) {
-	CASE(QK_QUERY);
-	CASE(QK_EVENT);
-	default:
-		return "QK_???";
-	}
-}
-
 static const char *
 status_to_str(int status)
 {
