@@ -1,4 +1,4 @@
-/*	$OpenBSD$	*/
+/*	$OpenBSD: mta_session.c,v 1.82 2015/12/12 20:02:31 gilles Exp $	*/
 
 /*
  * Copyright (c) 2008 Pierre-Yves Ritschard <pyr@openbsd.org>
@@ -321,8 +321,8 @@ mta_session_imsg(struct mproc *p, struct imsg *imsg)
 
 		if (resp_ca_cert->status == CA_FAIL) {
 			if (s->relay->pki_name) {
-				log_info("smtp-out: session %016"PRIx64
-				    ": disconnected (CA failure)", s->id);
+				log_info("smtp-out: Disconnecting session %016"PRIx64
+				    ": CA failure", s->id);
 				mta_free(s);
 				return;
 			}
@@ -343,6 +343,7 @@ mta_session_imsg(struct mproc *p, struct imsg *imsg)
 		if (ssl == NULL)
 			fatal("mta: ssl_mta_init");
 		io_start_tls(&s->io, ssl);
+
 		explicit_bzero(resp_ca_cert->cert, resp_ca_cert->cert_len);
 		free(resp_ca_cert->cert);
 		free(resp_ca_cert);
@@ -353,6 +354,7 @@ mta_session_imsg(struct mproc *p, struct imsg *imsg)
 		s = mta_tree_pop(&wait_ssl_verify, resp_ca_vrfy->reqid);
 		if (s == NULL)
 			return;
+
 		if (resp_ca_vrfy->status == CA_OK)
 			s->flags |= MTA_VERIFIED;
 		else if (s->relay->flags & F_TLS_VERIFY) {
@@ -361,6 +363,7 @@ mta_session_imsg(struct mproc *p, struct imsg *imsg)
 			mta_free(s);
 			return;
 		}
+
 		mta_io(&s->io, IO_TLSVERIFIED);
 		io_resume(&s->io, IO_PAUSE_IN);
 		io_reload(&s->io);
@@ -433,8 +436,7 @@ mta_free(struct mta_session *s)
 		fatalx("current task should have been deleted already");
 	if (s->datafp)
 		fclose(s->datafp);
-	if (s->helo)
-		free(s->helo);
+	free(s->helo);
 
 	relay = s->relay;
 	route = s->route;
@@ -551,10 +553,9 @@ mta_connect(struct mta_session *s)
 	else
 		schema = "smtp://";
 
-	log_info("smtp-out: session %016" PRIx64
-	    ": connecting to %s%s:%d (%s)",
-	    s->id, schema, sa_to_text(s->route->dst->sa),
-	    portno, s->route->dst->ptrname);
+	log_info("smtp-out: Connecting to %s%s:%d (%s) on session"
+	    " %016"PRIx64"...", schema, sa_to_text(s->route->dst->sa),
+	    portno, s->route->dst->ptrname, s->id);
 
 	mta_enter_state(s, MTA_INIT);
 	iobuf_xinit(&s->iobuf, 0, 0, "mta_connect");
@@ -718,8 +719,7 @@ mta_enter_state(struct mta_session *s, int newstate)
 		}
 
 		if (s->msgtried >= MAX_TRYBEFOREDISABLE) {
-			log_info("smtp-out: session %016"PRIx64
-			    "remote host seems to reject all mails",
+			log_info("smtp-out: Remote host seems to reject all mails on session %016"PRIx64,
 			    s->id);
 			mta_route_down(s->relay, s->route);
 			mta_enter_state(s, MTA_QUIT);
@@ -1149,6 +1149,8 @@ mta_io(struct io *io, int evt)
 	switch (evt) {
 
 	case IO_CONNECTED:
+		log_info("smtp-out: Connected on session %016"PRIx64, s->id);
+
 		if (s->use_smtps) {
 			io_set_write(io);
 			mta_start_tls(s);
@@ -1160,7 +1162,7 @@ mta_io(struct io *io, int evt)
 		break;
 
 	case IO_TLSREADY:
-		log_info("smtp-out: session %016"PRIx64": TLS started %s",
+		log_info("smtp-out: Started TLS on session %016"PRIx64": %s",
 		    s->id, ssl_to_text(s->io.ssl));
 		s->flags |= MTA_TLS;
 
@@ -1172,9 +1174,10 @@ mta_io(struct io *io, int evt)
 	case IO_TLSVERIFIED:
 		x = SSL_get_peer_certificate(s->io.ssl);
 		if (x) {
-			log_info("smtp-out: session %016"PRIx64
-			    ": server certificate verification %s",
-			    s->id, (s->flags & MTA_VERIFIED) ? "succeeded" : "failed");
+			log_info("smtp-out: Server certificate verification %s "
+			    "on session %016"PRIx64,
+			    (s->flags & MTA_VERIFIED) ? "succeeded" : "failed",
+			    s->id);
 			X509_free(x);
 		}
 
@@ -1257,10 +1260,9 @@ mta_io(struct io *io, int evt)
 		else
 			(void)strlcpy(s->replybuf, line, sizeof s->replybuf);
 
-
 		if (s->state == MTA_QUIT) {
-			log_info("smtp-out: session %016"PRIx64
-			    ": connection closed, %zu message%s sent.", s->id, s->msgcount,
+			log_info("smtp-out: Closing session %016"PRIx64
+			    ": %zu message%s sent.", s->id, s->msgcount,
 			    (s->msgcount > 1) ? "s" : "");
 			mta_free(s);
 			return;
@@ -1321,8 +1323,8 @@ mta_io(struct io *io, int evt)
 		else if (!(s->flags & (MTA_FORCE_TLS|MTA_FORCE_ANYSSL))) {
 			/* error in non-strict SSL negotiation, downgrade to plain */
 			if (s->flags & MTA_TLS) {
-				log_info("smtp-out: session %016"PRIx64
-				    ": error: opportunistic TLS failed, "
+				log_info("smtp-out: Error on session %016"PRIx64
+				    ": opportunistic TLS failed, "
 				    "downgrading to plain", s->id);
 				s->flags &= ~MTA_TLS;
 				s->flags |= MTA_DOWNGRADE_PLAIN;
@@ -1338,8 +1340,8 @@ mta_io(struct io *io, int evt)
 		log_debug("debug: mta: %p: TLS IO error: %s", s, io->error);
 		if (!(s->flags & (MTA_FORCE_TLS|MTA_FORCE_ANYSSL))) {
 			/* error in non-strict SSL negotiation, downgrade to plain */
-			log_info("smtp-out: session %016"PRIx64
-			    ": error: TLS failed, "
+			log_info("smtp-out: TLS Error on session %016"PRIx64
+			    ": TLS failed, "
 			    "downgrading to plain", s->id);
 			s->flags &= ~MTA_TLS;
 			s->flags |= MTA_DOWNGRADE_PLAIN;
@@ -1390,13 +1392,14 @@ mta_send(struct mta_session *s, char *fmt, ...)
 static ssize_t
 mta_queue_data(struct mta_session *s)
 {
-	char	*ln;
-	size_t	 len, q;
+	char	*ln = NULL;
+	size_t	 sz = 0, q;
+	ssize_t	 len;
 
 	q = iobuf_queued(&s->iobuf);
 
 	while (iobuf_queued(&s->iobuf) < MTA_HIWAT) {
-		if ((ln = fgetln(s->datafp, &len)) == NULL)
+		if ((len = getline(&ln, &sz, s->datafp)) == -1)
 			break;
 		if (ln[len - 1] == '\n')
 			ln[len - 1] = '\0';
@@ -1404,6 +1407,7 @@ mta_queue_data(struct mta_session *s)
 		    *ln == '.' ? "." : "", ln);
 	}
 
+	free(ln);
 	if (ferror(s->datafp)) {
 		mta_flush_task(s, IMSG_MTA_DELIVERY_TEMPFAIL,
 		    "Error reading content file", 0, 0);
@@ -1498,17 +1502,17 @@ mta_error(struct mta_session *s, const char *fmt, ...)
 	va_end(ap);
 
 	if (s->msgcount)
-		log_info("smtp-out: session %016"PRIx64
-		    ": error after %zu message%s sent: %s", s->id, s->msgcount,
+		log_info("smtp-out: Error on session %016"PRIx64
+		    " after %zu message%s sent: %s", s->id, s->msgcount,
 		    (s->msgcount > 1) ? "s" : "", error);
 	else
-		log_info("smtp-out: session %016"PRIx64 ": error: %s",
+		log_info("smtp-out: Error on session %016"PRIx64 ": %s",
 		    s->id, error);
 	/*
 	 * If not connected yet, and the error is not local, just ignore it
 	 * and try to reconnect.
 	 */
-	if (s->state == MTA_INIT && 
+	if (s->state == MTA_INIT &&
 	    (errno == ETIMEDOUT || errno == ECONNREFUSED)) {
 		log_debug("debug: mta: not reporting route error yet");
 		free(error);
@@ -1537,6 +1541,7 @@ mta_start_tls(struct mta_session *s)
 		certname = s->helo;
 		req_ca_cert.fallback = 1;
 	}
+
 	req_ca_cert.reqid = s->id;
 	(void)strlcpy(req_ca_cert.name, certname, sizeof req_ca_cert.name);
 	m_compose(p_lka, IMSG_MTA_TLS_INIT, 0, 0, -1,
