@@ -1,4 +1,4 @@
-/*	$OpenBSD: smtpd.h,v 1.455 2014/04/19 16:55:15 gilles Exp $	*/
+/*	$OpenBSD: smtpd.h,v 1.513 2016/02/21 15:17:25 gilles Exp $	*/
 
 /*
  * Copyright (c) 2008 Gilles Chehade <gilles@poolp.org>
@@ -35,6 +35,13 @@
 
 #include "rfc2822.h"
 
+#define CHECK_IMSG_DATA_SIZE(imsg, expected_sz) do {			\
+	if ((imsg)->hdr.len - IMSG_HEADER_SIZE != (expected_sz))	\
+		fatalx("smtpd: imsg %d: data size expected %zd got %zd",\
+	   	    (imsg)->hdr.type,					\
+	   	    (expected_sz), (imsg)->hdr.len - IMSG_HEADER_SIZE);	\
+} while (0)
+
 #ifndef SMTPD_CONFDIR
 #define SMTPD_CONFDIR		 "/etc"
 #endif
@@ -67,7 +74,7 @@
 #ifndef SMTPD_NAME
 #define	SMTPD_NAME		 "OpenSMTPD"
 #endif
-#define	SMTPD_VERSION		 "5.7.3p2"
+#define	SMTPD_VERSION		 "5.9.2p1"
 #define SMTPD_SESSION_TIMEOUT	 300
 #define SMTPD_BACKLOG		 5
 
@@ -80,7 +87,7 @@
 #define PATH_TEMPORARY		"/temporary"
 
 #ifndef	PATH_LIBEXEC
-#define	PATH_LIBEXEC		"/usr/libexec/smtpd"
+#define	PATH_LIBEXEC		"/usr/local/libexec/smtpd"
 #endif
 
 
@@ -106,8 +113,6 @@
 #define	F_EXT_DSN		0x400
 #define	F_RECEIVEDAUTH		0x800
 #define	F_MASQUERADE		0x1000
-#define	F_DSNNOTIFY_DISABLE    	0x2000
-#define	F_DSNRET_HEADERS	0x4000
 
 /* must match F_* for mta */
 #define RELAY_STARTTLS		0x01
@@ -119,6 +124,7 @@
 #define RELAY_MX		0x20
 #define RELAY_LMTP		0x80
 #define	RELAY_TLS_VERIFY	0x200
+
 #define MTA_EXT_DSN		0x400
 
 
@@ -579,6 +585,8 @@ struct listener {
 	char			 sendertable[PATH_MAX];
 
 	TAILQ_ENTRY(listener)	 entry;
+
+	int			 local;		/* there must be a better way */
 };
 
 struct smtpd {
@@ -631,11 +639,14 @@ struct smtpd {
 
 	time_t					 sc_uptime;
 
+	/* This is a listener for a local socket used by smtp_enqueue(). */
+	struct listener                         *sc_sock_listener;
+
 	TAILQ_HEAD(listenerlist, listener)	*sc_listeners;
 
 	TAILQ_HEAD(rulelist, rule)		*sc_rules;
 
-       	struct dict			       *sc_ca_dict;
+	struct dict			       *sc_ca_dict;
 	struct dict			       *sc_pki_dict;
 	struct dict			       *sc_ssl_dict;
 
@@ -648,8 +659,7 @@ struct smtpd {
 
 	char					sc_enqueue_filter[PATH_MAX];
 
-	char			       *sc_tls_ciphers;
-	char			       *sc_tls_curve;
+	char				       *sc_tls_ciphers;
 };
 
 #define	TRACE_DEBUG	0x0001
@@ -677,7 +687,7 @@ struct forward_req {
 	uint64_t			id;
 	uint8_t				status;
 
-	char				user[LINE_MAX];
+	char				user[SMTPD_VUSERNAME_SIZE];
 	uid_t				uid;
 	gid_t				gid;
 	char				directory[PATH_MAX];
@@ -1039,6 +1049,7 @@ struct msg {
 extern enum smtp_proc_type	smtpd_process;
 
 extern int verbose;
+extern int foreground_log;
 extern int profiling;
 
 extern struct mproc *p_control;
@@ -1131,7 +1142,6 @@ struct msg_walkinfo {
 
 /* aliases.c */
 int aliases_get(struct expand *, const char *);
-int aliases_virtual_check(struct table *, const struct mailaddr *);
 int aliases_virtual_get(struct expand *, const struct mailaddr *);
 int alias_parse(struct expandnode *, const char *);
 
@@ -1195,7 +1205,7 @@ void dns_imsg(struct mproc *, struct imsg *);
 
 
 /* enqueue.c */
-int		 enqueue(int, char **);
+int		 enqueue(int, char **, FILE *);
 
 
 /* envelope.c */
@@ -1217,6 +1227,18 @@ int expand_to_text(struct expand *, char *, size_t);
 RB_PROTOTYPE(expandtree, expandnode, nodes, expand_cmp);
 
 
+/* filter.c */
+void filter_postfork(void);
+void filter_configure(void);
+void filter_connect(uint64_t, const struct sockaddr *,
+    const struct sockaddr *, const char *, const char *);
+void filter_mailaddr(uint64_t, int, const struct mailaddr *);
+void filter_line(uint64_t, int, const char *);
+void filter_eom(uint64_t, int, size_t);
+void filter_event(uint64_t, int);
+void filter_build_fd_chain(uint64_t, int);
+
+
 /* forward.c */
 int forwards_get(int, struct expand *);
 
@@ -1230,10 +1252,12 @@ void imsgproc_set_write(struct imsgproc *);
 void imsgproc_set_read_write(struct imsgproc *);
 void imsgproc_reset_callback(struct imsgproc *, void (*)(struct imsg *, void *), void *);
 
+
 /* limit.c */
 void limit_mta_set_defaults(struct mta_limits *);
 int limit_mta_set(struct mta_limits *, const char*, int64_t);
 
+
 /* lka.c */
 pid_t lka(void);
 
@@ -1247,22 +1271,15 @@ void lka_session_forward_reply(struct forward_req *, int);
 void vlog(int, const char *, va_list);
 void logit(int, const char *, ...) __attribute__((format (printf, 2, 3)));
 
+
 /* mda.c */
 void mda_postfork(void);
 void mda_postprivdrop(void);
 void mda_imsg(struct mproc *, struct imsg *);
 
 
-/* filter.c */
-void filter_postfork(void);
-void filter_configure(void);
-void filter_connect(uint64_t, const struct sockaddr *,
-    const struct sockaddr *, const char *, const char *);
-void filter_mailaddr(uint64_t, int, const struct mailaddr *);
-void filter_line(uint64_t, int, const char *);
-void filter_eom(uint64_t, int, size_t);
-void filter_event(uint64_t, int);
-void filter_build_fd_chain(uint64_t, int);
+/* makemap.c */
+int makemap(int, char **);
 
 
 /* mailaddr.c */
@@ -1271,6 +1288,7 @@ void maddrmap_init(struct maddrmap *);
 void maddrmap_insert(struct maddrmap *, struct maddrnode *);
 void maddrmap_free(struct maddrmap *);
 
+
 /* mproc.c */
 int mproc_fork(struct mproc *, const char*, char **);
 void mproc_init(struct mproc *, int);
@@ -1334,6 +1352,7 @@ struct mta_task *mta_route_next_task(struct mta_relay *, struct mta_route *);
 const char *mta_host_to_text(struct mta_host *);
 const char *mta_relay_to_text(struct mta_relay *);
 
+
 /* mta_session.c */
 void mta_session(struct mta_relay *, struct mta_route *);
 void mta_session_imsg(struct mproc *, struct imsg *);
@@ -1368,6 +1387,7 @@ int queue_envelope_update(struct envelope *);
 int queue_envelope_walk(struct envelope *);
 int queue_message_walk(struct envelope *, uint32_t, int *, void **);
 
+
 /* ruleset.c */
 struct rule *ruleset_match(const struct envelope *);
 
@@ -1414,7 +1434,7 @@ int fork_proc_backend(const char *, const char *, const char *);
 
 /* ssl_smtpd.c */
 void   *ssl_mta_init(void *, char *, off_t, const char *);
-void   *ssl_smtp_init(void *);
+void   *ssl_smtp_init(void *, int);
 
 
 /* stat_backend.c */
@@ -1474,6 +1494,7 @@ const char *sockaddr_to_text(struct sockaddr *);
 const char *mailaddr_to_text(const struct mailaddr *);
 const char *expandnode_to_text(struct expandnode *);
 
+
 /* util.c */
 typedef struct arglist arglist;
 struct arglist {
@@ -1497,7 +1518,6 @@ void xlowercase(char *, const char *, size_t);
 int  uppercase(char *, const char *, size_t);
 uint64_t generate_uid(void);
 int availdesc(void);
-int ckdir_quiet(const char *, mode_t, uid_t, gid_t);
 int ckdir(const char *, mode_t, uid_t, gid_t, int);
 int rmtree(char *, int);
 int mvpurge(char *, char *);
@@ -1524,6 +1544,7 @@ int base64_decode(char const *, unsigned char *, size_t);
 int  waitq_wait(void *, void (*)(void *, void *, void *), void *);
 void waitq_run(void *, void *);
 
+
 /* runq.c */
 struct runq;
 
